﻿using FL.Data;
using FL.Nodes.GameMain;
using System.Collections.Generic;
using UnityEngine;
using XGame.Database;
using XGame.Framework.Asyncs;
using XGame.Framework.Interfaces;
using XGame.Framework.Map;

namespace FL.Map
{
    public class TestMainMapEntitiesCtrl : EntityController<TestMainMapVM>, IUpdate, ILateUpdate
    {
        private PlayerEntityView _player;
        private Dictionary<int, MonsterEntityView> _monsterEntityMap = new Dictionary<int, MonsterEntityView>();
        public override void OnEnable(object intent)
        {
            EventSingle.Instance.AddListener(EventDefine.GameMainMapChapterChanged, OnChapterChanged);
        }
        public override void OnDisable()
        {
            EventSingle.Instance.RemoveListener(EventDefine.GameMainMapChapterChanged, OnChapterChanged);
            Context.Asset.Recycle(_player);
            foreach(var monster in _monsterEntityMap.Values)
            {
                Context.Asset.Recycle(monster);
            }
            _monsterEntityMap.Clear();
        }

        void IUpdate.Update(int millisecond)
        {
            if (_player != null)
            {
                var dir = 0;
                if (Input.GetKey(KeyCode.UpArrow))
                {
                    dir = 1;
                }
                else if (Input.GetKey(KeyCode.DownArrow))
                {
                    dir = -1;
                }
                else
                {
                    return;
                }
                var distanceY = PlayerEntityData.Instance.moveSpeed * dir * millisecond * 0.001f;
                _player.Move(distanceY);
            }


        }
        void ILateUpdate.LateUpdate(int millisecond)
        {
            VM.UpdateMapMoveTime(millisecond);
            VM.Camera.OnLateUpdate(millisecond);
        }

        #region 事件监听
        private void OnChapterChanged(int eventId, object args)
        {
            var dto = (MapChapterChangeDto)args;
            var asyncGroup = new AsyncGroup();
            if (_player == null)
            { // 第一波怪物，玩家不存在时
                var playerAsync = Context.Asset.LoadAsync<EntityBehaviour>("Player");
                playerAsync.Join(asyncGroup);
                playerAsync.On(_ =>
                {
                    var behaviour = playerAsync.GetResult();
                    if (behaviour == null)
                    {
                        return;
                    }
                    var entity = Context.Asset.CreateEntity<PlayerEntityView>(behaviour);
                    behaviour.transform.SetParent(VM.PlayerRoot, false);
                    entity.LocalPosition = dto.playerBirth;
                    entity.Enable(null);
                    _player = entity;
                    VM.Camera.Target = behaviour.transform;
                });
            }
            var monsterCount = dto.monsterInfos?.Length ?? 0;
            if (monsterCount > 0)
            { // 创建怪物实例
                for(var i = 0; i < monsterCount; i++)
                {
                    var monsterInfo = dto.monsterInfos[i];
                    var monsterTable = MonsterTableRepo.Get(monsterInfo.tableId);
                    var monsterAsync = Context.Asset.LoadAsync<EntityBehaviour>(monsterTable.Model);
                    monsterAsync.Join(asyncGroup);
                    monsterAsync.On(_ =>
                    {
                        var behaviour = monsterAsync.GetResult();
                        if (behaviour == null)
                        {
                            return;
                        }
                        var entity = Context.Asset.CreateEntity<MonsterEntityView>(behaviour);
                        entity.Info = monsterInfo;
                        behaviour.transform.SetParent(VM.EnemyRoot, false);
                        entity.LocalPosition = monsterInfo.position;
                        entity.Enable(null);
                        _monsterEntityMap.Add(monsterInfo.instanceId, entity);
                    });
                }
                asyncGroup.On(_ =>
                {
                    //Entities加载完成，开始移动
                    _player.MoveTo(dto.battlePosition, PlayerEntityData.Instance.moveSpeed);
                    for(var i = 0; i < monsterCount; i++)
                    {
                        var monsterInfo = dto.monsterInfos[i];
                        var monsterTable = MonsterTableRepo.Get(monsterInfo.tableId);
                        var entity = _monsterEntityMap[monsterInfo.instanceId];
                        entity.MoveTo(dto.battlePosition, monsterTable.Speed);
                    }
                });
                asyncGroup.End();
            }
        }
        #endregion
    }
}