﻿using FL.Data;
using FL.Map;
using System.Collections.Generic;
using UnityEngine;
using XGame.Database;
using XGame.Framework.Asyncs;
using XGame.Framework.Loadable;
using XGame.Framework.Nodes;
using XGame.Framework.Time;

namespace FL.Nodes.GameMain
{
    public class MapBusiness : BusinessBase, ILoadAsyncHandle
    {
        /// <summary>
        /// 怪物的实例Id，自增
        /// </summary>
        private static int Monster_Id_Add = 0;
        private int _chapterId;
        private EMapSchedule _schedule;
        private ITimer _monsterTimer;
        /// <summary>
        /// 怪物的实例对象
        /// </summary>
        private Dictionary<int, MapMonsterInfo> _monsterMap;
        public override void OnEnable(object intent)
        {
            _chapterId = PlayerEntityData.Instance.chapterId;
            _schedule = EMapSchedule.LoadingMap;
            var chapter = ChapterTableRepo.Get(_chapterId);
            var map = MapTableRepo.Get(chapter.Map);
            var mapAsync = Context.Map.RunAsync(MapKeys.NameToKey(map.AssetName));
            mapAsync.On(_ =>
            {
                _schedule = EMapSchedule.Monster1;
                var dto = new MapChapterChangeDto();
                dto.chapterId = _chapterId;
                //玩家出生点
                dto.playerBirth = map.PlayerBirth.ToVector3();
                //战斗位置
                dto.battlePosition = map.BattlePositions[chapter.Part - 1].ToVector3();
                //怪物数据
                var monsterInfos = GenMonsters();
                AddMonsters(monsterInfos);
                dto.monsterInfos = monsterInfos;
                //定时器
                if (chapter.Part_type == 0)
                { // 普通节点，增加定时器
                    _monsterTimer = Context.Time.AddDelayLooperTimer(chapter.Interval, chapter.Interval, () =>
                    {

                    });
                }
                EventSingle.Instance.Notify(EventDefine.GameMainMapChapterChanged, dto);
            });
        }
        public override void OnDisable()
        {
            _monsterTimer?.Cancel();
            _monsterTimer = null;
            _monsterMap?.Clear();
            Context.Map.Close(MapKeys.TestMainMap);
        }

        void ILoadAsyncHandle.OnLoadAsync(IAsyncGroup group)
        {
            ChapterTableRepo.LoadAsync().Join(group);
            MapTableRepo.LoadAsync().Join(group);
            MonsterTableRepo.LoadAsync().Join(group);
        }

        void ILoadAsyncHandle.OnUnloadAsync(IAsyncGroup group)
        {
            ChapterTableRepo.Unload();
            MapTableRepo.Unload();
            MonsterTableRepo.Unload();
        }

        private MapMonsterInfo[] GenMonsters()
        {
            var chapter = ChapterTableRepo.Get(_chapterId);
            var map = MapTableRepo.Get(chapter.Map);
            Vector3 monsterPosition = map.MonsterPositions[chapter.Part - 1].ToVector3();
            int[] monsterRefresh = null;
            switch (_schedule)
            {
                case EMapSchedule.Monster1:
                    monsterRefresh = chapter.Monster_refresh1;
                    break;
                case EMapSchedule.Monster2: 
                    monsterRefresh = chapter.Monster_refresh2;
                    break;
                case EMapSchedule.Monster3: 
                    monsterRefresh = chapter.Monster_refresh3;
                    break;
                case EMapSchedule.Monster4: 
                    monsterRefresh = chapter.Monster_refresh4;
                    break;
                case EMapSchedule.Monster5: 
                    monsterRefresh = chapter.Monster_refresh5;
                    break;
                default:
                    break;
            }
            var monsterCount = (monsterRefresh != null && monsterRefresh.Length > 0) ? monsterRefresh[1] : 0;
            if (monsterCount > 0)
            { // 数量0-3
                var xStart = monsterCount == 1 ? 0 : -2;
                var xStep = monsterCount == 2 ? 4 : 2;
                var monsterInfos = new MapMonsterInfo[monsterCount];
                for(var  i = 0; i < monsterCount; i++)
                {
                    var position = monsterPosition;
                    position.x = xStart + xStep * i;
                    monsterInfos[i] = new MapMonsterInfo()
                    {
                        tableId = monsterRefresh[0],
                        instanceId = Monster_Id_Add++,
                        position = position
                    };
                }
                return monsterInfos;
            }
            return null;
        }
        private void AddMonsters(MapMonsterInfo[] monsterInfos)
        {
            if (monsterInfos == null || monsterInfos.Length == 0) return;
            _monsterMap = new Dictionary<int, MapMonsterInfo>();
            foreach (var monsterInfo in monsterInfos)
            {
                _monsterMap.Add(monsterInfo.instanceId, monsterInfo);
            }
        }
    }
}